/**************************************************************************
Workspace: IAR+SPL
Chip type: STM8S003K3 | STM8S103K3
Project  : time relay/caps psu
Date     : Mar, 2023
Version  : 0
****************************************************************************/
#include <stdbool.h>
#include <stdlib.h>

#include "stm8s.h"
#include "iostm003.c"
#include "bitset.c"
#include "stm8s_itc.c"
#include "stm8s_flash.c"

#include "defines.c"
#include "eeprom_rtns.c"
#include "periph_init.c"
#include "scr_cntrl.c"
#include "adc_rtns.c"
#include "misc.c"
#include "kwork.c"
#include "crc_rtns.c"
#include "interrupts.c"

void devcon(const u16, const u16);
void main()
{
 cfg.gpio();

/*
 //crc check -   ,  16 
 {
  cfg_spi_for_crc();

  u8 crc = get_crc((u8*)flash_begin, (u8*)flash_end);

  if(Checksum == 0) store_flash_checksum(crc);
  else if(Checksum != crc)
  {
   LED_RED(true);

 #ifndef CRC_DBG
   ENDLESS;
 #endif
  }
  else
  {
   LED_RED(false);
  }
 }
*/

 cfg.clk();
 cfg.t1();
 cfg.t2();
 cfg.adc();

 __enable_interrupt();

 ENDLESS
 {
  u16 Vcc = 0;
  u16 Vbat;

  //    ,   
  if(ADC1->CSR & ADC1_CSR_EOC)
  {
   get_Vcc(&Vcc);
   get_Vbat(&Vbat, Vcc);

   stop_adc();
  }

  // 
  if(!Tim[KBD])
  {
   Tim[KBD] = 1;

   scan_keys();
  }

  devcon(Vbat, Vcc);

  __wait_for_interrupt();
 }
}

void devcon(const u16 Vbat, const u16 Vcc)
{
 static bool itr;
 static enum {start, work_on_mp, work_on_bat, setup, standby, error_state} dev_st = start, dev_st_tmp;
 static enum {
              no_error,
              vd_error
             } error = no_error;
 static bool btn_trg;
 static bool output_trg;

 m:

 //     
 if(error != no_error) dev_st = error_state;
 if(dev_st != dev_st_tmp)
 {
  dev_st_tmp = dev_st;

  LED_GRN(false);
  LED_RED(false);

  itr = 0;
 }

 switch (dev_st)
 {
  case start:
  {
   static u16 Vcc_tmp;

   if(!itr)
   {
    itr = 1;

    scr_off();
    charge_bat(false);
    bat_feeding(false);

    output_trg = false;

    LED_GRN(true);

    Vcc_tmp = 0;
   }

   //     
   if(Vcc)
   {
    //   
    if(Vcc_tmp)
    {
     //Vcc 
     if(Vcc >= Vcc_tmp - Vcc_tmp/8 && Vcc <= Vcc_tmp + Vcc_tmp/8)
     {
      //        
      if (Vcc > VCC_MIN) {dev_st = setup;   goto m;}
      else               {dev_st = standby; goto m;}
     }
    }
    else Vcc_tmp = Vcc;
   }
  }
  break;

  case work_on_mp:
  {
   if(!itr)
   {
    itr = 1;

    bat_feeding(false);
   }

   //     
   {
    if(Btn[0]==rls)
    {
     //  ,    ./.
     if(btn_trg == true)
     {
      btn_trg = false;

      output_trg ^= true;
     }
    }
    //        
    else if(Btn[0]==hold)
    {
     dev_st = setup;
     btn_trg = false;
    }
    else btn_trg = true;
   }

   // ,   
   if(!Tim[MPC])
   {
    Tim[MPC] = TSEC;

    //            
    if(output_trg==true) {if(get_scr_status()==false) scr_on();}
    //          
    else
    {
     static bool trg = false;                 //   
     static u16 stc_on[SW_PER];               //    
     static u16 stc_off[SW_PER];              //     

     //     
     if(trg==false)
     {
      trg = true;

      for(u8 i=0; i<SW_PER; i++)
      {
       u16 tmp;

       tmp        = Switching_time_data[i].time_on.hours *60;
       stc_on[i]  = tmp + Switching_time_data[i].time_on.minutes;
       tmp        = Switching_time_data[i].time_off.hours*60;
       stc_off[i] = tmp + Switching_time_data[i].time_off.minutes;
      }
     }

     //    
     u16 rtc;
     {
      __istate_t s = __get_interrupt_state();
      __disable_interrupt();

      rtc  = Main.hours*60;
      rtc += Main.minutes;

      __set_interrupt_state(s);
     }

     //        (   )
     if(D_trg==true)
     {
      D_trg = false;

      for(u8 i=0; i<SW_PER; i++)
      {
       //     
       if(Switching_time_data[i].random_on == true)
       {
        u16 tmp   = Switching_time_data[i].time_on.hours *60;
        stc_on[i] = tmp + Switching_time_data[i].time_on.minutes;

        stc_dt_corr(&Switching_time_data[i], &stc_on[i]);
       }

       if(Switching_time_data[i].random_off==true)
       {
        u16 tmp    = Switching_time_data[i].time_off.hours*60;
        stc_off[i] = tmp + Switching_time_data[i].time_off.minutes;

        stc_dt_corr(&Switching_time_data[i], &stc_off[i]);
       }
      }
     }

     //      
     bool on_off = false;

     for(u8 i=0; i<SW_PER; i++)
     {
      //   
      {
       //      
       //         /
       if(stc_on[i] < stc_off[i])
       {
        if(rtc >= stc_on[i] && rtc < stc_off[i]) on_off = true;
       }
       //     
       else
       {
        if(rtc >= stc_on[i] || rtc < stc_off[i]) on_off = true;
       }
      }
     }

     //      
     if(on_off != get_scr_status()) scr_control(on_off);
    }
   }

   // |   
   if(!Tim[LED])
   {
    static bool trg;

    Tim[LED] = TSEC/2;

    trg ^= true;

    //      /
    if(output_trg==true)
    {
     LED_GRN(trg);
     LED_RED(!trg);
    }
    //           
    //     
    else
    {
     if(get_scr_status()==false) LED_GRN(trg);
     else                        LED_RED(trg);
    }
   }

   //    
   if(Vcc)
   {
    //          
    if(Vcc < Vbat + DVD && Vcc < VCC_MIN)
    {
     dev_st = work_on_bat;
     bat_feeding(true);
    }
    //  
    else
    {
     if(Vbat > VBT_TOP) charge_bat(false);
     else               charge_bat(true);
    }
   }
  }
  break;

  case work_on_bat:
  {
   if(!itr)
   {
    itr = 1;

    scr_off();
    bat_feeding(true);
    charge_bat(true);

    Tim[MPC] = TSEC*2;
   }

   if(!Tim[MPC])
   {
    if(Vcc)
    {
     //       
     if(Vcc > Vbat && Vcc > VCC_MIN) dev_st = work_on_mp;
     //       
     else if(Vbat < VBT_DN) dev_st = standby;
    }
   }
  }
  break;

  case setup:
  {
   if(!itr)
   {
    //    -    ,    
    static u8 stage = 0;

    if(!Tim[COM])
    {
     Tim[COM] = TSEC/8;

     q:
     if(!stage)
     {
      stage++;
      LED_RED(true);

      scr_off();
      charge_bat(false);
      bat_feeding(false);
     }
     else if(stage==1)
     {
      stage++;
      LED_GRN(true);
     }
     else
     {
      #define CYCN 4

      static u8 cnt = CYCN;

      stage = 0;
      LED_GRN(false);

      if(cnt)
      {
       cnt--;
       goto q;
      }
      else
      {
       cnt = CYCN;
       itr = true;

       Tim[COM] = 0;
      }
     }
    }
   }
   else if(Btn[0]==rls)
   {
    __istate_t s = __get_interrupt_state();
    __disable_interrupt();

    Main.hours   = EE_storage.syncro.hours;
    Main.minutes = EE_storage.syncro.minutes;

    __set_interrupt_state(s);

    dev_st = work_on_mp;
   }
  }
  break;

  case standby:
  {
   if(!itr)
   {
    itr = 1;

    scr_off();
    bat_feeding(false);
    charge_bat(true);

    CLK->PCKENR1 = 0;          // 

    CLK->SWR  = 0xe1;          //HSI as master clock source
    CLK->SWCR = CLK_SWCR_SWEN; //  
    CLK->ECKR = 0;             //  

    // awu
    cfg.awu();

    halt();
   }

   //           
   ENDLESS
   {
    start_adc();
    start_adc();

    while (!(ADC1->CSR & ADC1_CSR_EOC)) continue;

    u16 Vcc;

    get_Vcc(&Vcc);

    stop_adc();

    if(Vcc < VCC_MIN) halt();
    else WWDG->CR = WWDG_CR_WDGA; //software reset
   }
  }
  break;

  case error_state:
  {
   static int ind_stage;
   static bool ind_trg;

   if(!itr)
   {
    ind_stage = 0;
    ind_trg = false;

    Tim[ERR] = TSEC;
   }

   /*
     
        
        2      
   */
   if(!Tim[ERR])
   {
    //   
    if(ind_trg == false)
    {
     ind_trg = true;

     // -  
     if(ind_stage == 0)
     {
      Tim[ERR] = TSEC;
      ind_stage ++;

      LED_RED(1);
     }
     // -     
     else if(ind_stage < (int)error+1)
     {
      Tim[ERR] = TSEC/2;
      ind_stage ++;

      LED_GRN(1);
     }
     else
     {
      ind_stage = 0;
      ind_trg = false;

      Tim[ERR] = TSEC*2;
     }
    }
    //    
    else
    {
     ind_trg = false;
     Tim[ERR] = TSEC/2;

     LED_RED(0);
     LED_GRN(0);
    }
   }

   if(dev_st != error_state) error = no_error;
  }
  break;

  default: break;
 }
}

